/*******************************************************************
*
*  DESCRIPTION: Atomic Model Balancer
*
*  AUTHOR: Changshuang Lin 
*
*  EMAIL: clin@sce.carleton.ca
*  DATE: 16/10/2003
*
*******************************************************************/

/** include files **/
#include "balancer.h"      // class Balancer
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: Queue
* Description: 
********************************************************************/
Balancer::Balancer( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, out1( addOutputPort( "out1" ) )
, out2( addOutputPort( "out2" ) )
, out3( addOutputPort( "out3" ) )
, dispatchTime( 0, 0, 0, 010 )
{
	string time( MainSimulator::Instance().getParameter( description(), "dispatch" ) ) ;

	if( time != "" )
		dispatchTime = time ;
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Balancer::initFunction()
{
	job_queue.erase( job_queue.begin(), job_queue.end() ) ;
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Balancer::externalFunction( const ExternalMessage &msg )
{
	int new_pid =   (int) msg.value();
        cout <<msg.time(). asString() << "	Balancer receives job# " <<new_pid << endl; 
	job_queue.push_back( msg.value() ) ;
       
     	// if this is the only job in the job queue, start dispatching
    	if( job_queue.size() == 1 ) {
 		int pid = (int) job_queue.front();  	
           	cout << msg.time().asString() <<"	Balancer starts dispatching job# " << pid  <<endl;
		holdIn( active, dispatchTime );           
        }

     	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Balancer::internalFunction( const InternalMessage &msg )
{
        // if there is any job in the queue, start dispatching  
      	if  (job_queue.size() > 0 ) {
            	int pid = (int) job_queue.front();  	
           	cout << msg.time().asString() <<"	Balancer starts dispatching job# " << pid  <<endl;
           	holdIn( active, dispatchTime );
       }
       else  
           	passivate();

      	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Balancer::outputFunction( const InternalMessage &msg )
{
        int pid = (int) job_queue.front();
        // round-robin dispatching
	if (pid % 3 == 0) {
		cout << msg.time().asString()<<"	Balancer sends job# " << pid << "  to server 1" <<endl;          	
          	sendOutput( msg.time(), out1, pid ) ;
        }
        else if (pid % 3 == 1) {
          	cout << msg.time().asString()<<"	Balancer sends job# " << pid << "  to server 2" <<endl;               	
          	sendOutput( msg.time(), out2, pid ) ;
        }
        else {
 		cout << msg.time().asString()<<"	Balancer sends job# " << pid << "  to server 3" <<endl;              	
          	sendOutput( msg.time(), out3, pid ) ;
        }
 
    	//pop up the job
       	job_queue.pop_front() ;   
       	return *this ;
}
