/*******************************************************************
*
*  DESCRIPTION: Atomic Model DBServer
*
*  AUTHOR: Changshuang Lin 
*
*  EMAIL: clin@sce.carleton.ca
*  DATE: 16/10/2003
*
*******************************************************************/

/** include files **/
#include "dbserver.h"      // class DBServer
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: DBServer
* Description: 
********************************************************************/
DBServer::DBServer( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, out1( addOutputPort( "out1" ) )
, out2( addOutputPort( "out2" ) )
, out3( addOutputPort( "out3" ) )
, dbTime( 0, 0, 0, 010 )
,jobs_done(0)
{
	string time( MainSimulator::Instance().getParameter( description(), "process" ) ) ;

	if( time != "" )
		dbTime = time ;
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &DBServer::initFunction()
{
	job_queue.erase( job_queue.begin(), job_queue.end() ) ;
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &DBServer::externalFunction( const ExternalMessage &msg )
{
	server_id =   (int) msg.value();
        cout << msg.time(). asString() <<"	DBServer receives a job from server "<< server_id<<endl; 
	job_queue.push_back( msg.value() ) ;
       
     	// if this is the only job in the job queue, start dispatching
    	if( job_queue.size() == 1 ) {
 		server_id = (int) job_queue.front();  	
           	cout << msg.time().asString() << "	DBServer starts processing job from server " << server_id << endl;
		holdIn( active, dbTime );           
        }

     	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &DBServer::internalFunction( const InternalMessage &msg )
{
        // if there is any job in the queue, start dispatching  
      	if  (job_queue.size() > 0 ) {
            	server_id = (int) job_queue.front();  	
		cout << msg.time().asString() << "	DBServer starts processing job from server " << server_id << endl;           	
           	holdIn( active, dbTime );
       }
       else  
           	passivate();

      	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &DBServer::outputFunction( const InternalMessage &msg )
{
       	server_id = (int) job_queue.front();
	cout << msg.time().asString() << "	DBServer sends job back to server " << server_id <<endl;
	if (server_id == 1)    	sendOutput( msg.time(), out1, server_id ) ;
        else if (server_id == 2)      	sendOutput( msg.time(), out2, server_id ) ;
        else if (server_id == 3)       	sendOutput( msg.time(), out3, server_id ) ;
        else      	cout << msg.time().asString() << "   unknown server" <<endl;          	
               
        //pop up the job
       	job_queue.pop_front() ;
	jobs_done++;
	cout <<  msg.time().asString() <<"	Number of jobs done = " << jobs_done << "  ******* " << endl;   
       	return *this ;
}
