/*******************************************************************
*
*  DESCRIPTION: Atomic Model Server
*
*  AUTHOR: Changshuang Lin 
*
*  EMAIL: clin@sce.carleton.ca
*  DATE: 16/10/2003
*
*******************************************************************/

/** include files **/
#include <math.h>        // fabs( ... )
#include "server.h"           // base header
#include "message.h"       // InternalMessage ....
#include "distri.h"        // class Distribution
#include "mainsimu.h"      // class MainSimulator
#include "strutil.h"       // str2float( ... )
//#include <stdlib.h>    // atoi

/*******************************************************************
* Function Name: Server
* Description: constructor
********************************************************************/
Server::Server( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, done( addInputPort( "done" ) )
, out( addOutputPort( "out" ) )
, db( addOutputPort( "db" ) )
,id(0)
{
	try
	{
		dist = Distribution::create( MainSimulator::Instance().getParameter( description(), "distribution" ) );
		server_id = MainSimulator::Instance().getParameter( description(), "server_id" );
		if (server_id == "1") id =1;
 		else if (server_id == "2") id =2;
                else if (server_id == "3") id =3;  
                //cout << "ID = " << id << endl;
		MASSERT( dist );

		for ( register int i = 0 ; i < dist->varCount() ; i++ )
		{
			string parameter( MainSimulator::Instance().getParameter( description(), dist->getVar(i) ) ) ;
			dist->setVar( i, str2float( parameter ) ) ;
		}
	} catch( InvalidDistribution &e )
	{
		e.addText( "The model " + description() + " has distribution problems!" ) ;
		e.print(cerr);
		MTHROW( e ) ;
	} catch( MException &e )
	{
		MTHROW( e ) ;
	}
}

/*******************************************************************
* Function Name: externalFunction
* Description: the Server receives one job
********************************************************************/
Model &Server::externalFunction( const ExternalMessage &msg )
{
	pid = static_cast< int >( msg.value() ) ;
        if (msg.port()==in) {
	        cout << msg.time(). asString() <<"	Server " << id << " receives job# " <<pid <<endl; 
		job_queue.push_back( msg.value() ) ;          
     		// if this is the only job in the job queue, start processing
    		if( job_queue.size() == 1 && (!waiting) ) {
			cout << msg.time().asString()<<" 	Server " << id << " starts processing job# " << pid << endl; 			
			Time t = Time( /*0,0,5,0 */static_cast<float>( fabs( distribution().get() ) )  ) ;
           		holdIn( active, t) ; 			
        	}
   	}
	else if (msg.port()==done && waiting) {
		holdIn( active, Time::Zero) ;			
	}
	
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Server::internalFunction( const InternalMessage &msg )
{
       	//change the waiting state
    	waiting = !waiting; 
     
	// if there is any job in the queue, start processing  
      	if  (job_queue.size() > 0 && (!waiting) ) {
            	int pid1 = (int) job_queue.front();  	
           	cout << msg.time().asString()<<" 	Server " << id << " starts processing job# " << pid1 << endl;
		Time t = Time( /*0,0,5,0 */static_cast<float>( fabs( distribution().get() ) )  ) ;
           	holdIn( active, t) ;		
       	}
       	else  
		passivate();

	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Server::outputFunction( const InternalMessage &msg )
{

	if (waiting) {
		cout << msg.time().asString() << "	Server "<< id << " finishes job# " << pid_sent <<endl;
       		sendOutput( msg.time(), out, pid_sent ) ;
		//waiting = false;
	}

	else {
		pid_sent = (int) job_queue.front();
		cout << msg.time().asString() << "    Server " << id << " sends job# " << pid_sent << " to database server."<<endl;
       		
                // server_id indicates the retuned server ID 
                sendOutput( msg.time(), db, id) ;
       		//pop up the job
       		job_queue.pop_front() ;  
		//waiting = true;
     	}
	
	return *this;
}

Server::~Server()
{
	delete dist;
}